#include "../Framework/Framework.h"

#define SIZE 32

class MainApp : public OpenGLApp {
protected:
	TextureID entropy0, entropy1;
	ShaderID noise;

public:
	void selectPixelFormat(PixelFormat &pixelFormat);

	bool load();
	bool drawFrame();
};

void MainApp::selectPixelFormat(PixelFormat &pixelFormat){
	pixelFormat.redBits   = 8;
	pixelFormat.greenBits = 8;
	pixelFormat.blueBits  = 8;
	pixelFormat.alphaBits = 0;

	pixelFormat.depthBits = 16;
}

bool MainApp::load(){

	String supported, missing;

    if (GL_ARB_fragment_program) supported += "GL_ARB_fragment_program\n"; else missing += "GL_ARB_fragment_program\n";

    if (missing.getLength() > 0){
        ALERT("The following required OpenGL extensions are supported:\n\n" + supported +
				   "\nToo bad, these required OpenGL extensions aren't supported by the graphic card and driver you're using:\n\n" + missing +
                   "\nThis application will now exit.", "Missing extensions");
        return false;
    }
	
	noise = renderer->addFragmentShader(GL_FRAGMENT_PROGRAM_ARB, "Noise.fsh");

	entropy0 = renderer->addEntropy2D(SIZE, TEX_NOMIPMAPPING | TEX_NOLINEARFILTER);
	entropy1 = renderer->addEntropy2D(SIZE, TEX_NOMIPMAPPING | TEX_NOLINEARFILTER);

	textTexture = renderer->addTexture("../Textures/font2.dds");

	if (glGetError() != GL_NO_ERROR){
		char *str = (char *) glGetString(GL_PROGRAM_ERROR_STRING_ARB);
		ALERT(str, "Error");
	}


	return true;
}

bool MainApp::drawFrame(){

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(0,1,0,1,-1,1);

	renderer->setTextures(entropy0, entropy1);
	renderer->setFragmentShader(noise);
	renderer->apply();

	glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, 0, SIZE, 1.0f / SIZE, 0.5f / SIZE, 0);
	glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, 1, currTime, currTime, currTime, currTime);

	glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex2f(0,0);

		glTexCoord2f(1,0);
		glVertex2f(1,0);

		glTexCoord2f(1,1);
		glVertex2f(1,1);

		glTexCoord2f(0,1);
		glVertex2f(0,1);
	glEnd();


	return true;
}

DECLARE_MAIN_APP(MainApp)
